package org.tbasket

import io.getquill.{Literal, SqliteDialect}
import io.getquill.context.qzio.ZioContext
import org.tbasket.api.Endpoint
import org.tbasket.handler.{IncrementHandler, LoginHandler}
import org.tbasket.auth.UserSessionHandler

import java.util.Properties

object EndpointSetup:

  private final val EndpointPort = "endpoint.port"
  private final val EndpointPortDefault = "48485"

  def setupEndpoint(config: Properties, db: ZioContext[SqliteDialect, Literal]): Endpoint =
    Main.LOG.debug("Initializing API endpoint...")
    
    
    val endpoint = createEndpoint(config)
    endpoint.bind("counter")(IncrementHandler)
    endpoint.bind("login")(new LoginHandler(sessions, db))
    endpoint

  private def createEndpoint(config: Properties): Endpoint =
    val port = config
      .getProperty(EndpointPort, EndpointPortDefault) match
      case s"$port" if port.toIntOption.isDefined => port.toInt
      case v =>
        throw new InternalBasketServerException(
          s"$EndpointPort property value is wrong: $v must be integer"
        )
    new Endpoint(port)
