package org.tbasket

import org.apache.logging.log4j.LogManager
import org.tbasket.db.Database
import zio.*

import java.lang
import java.nio.file.{Files, Path}
import java.util.Properties
import scala.io.StdIn
import scala.util.{Failure, Success}
import scala.util.control.NonFatal

object Main extends ZIOAppDefault:
  final val LOG = LogManager.getLogger("Core")

  override def run: ZIO[Main.Environment with ZIOAppArgs with Scope, Any, Any] =
    val config = retrieveConfig
    val db = new Database(config)
    EndpointSetup.setupEndpoint(config, db.layer).run

  private def retrieveConfig: Properties =
    val configFile = Path.of("server.properties")
    if Files.notExists(configFile) then
      val in = getClass.getResourceAsStream("/server.properties")
      Files.writeString(configFile, new String(in.readAllBytes()))
    val in = Files.newInputStream(configFile)
    val properties = new Properties()
    properties.load(in)
    properties

  // add a shutdown hook to log when the server is about to get killed
  lang.Runtime.getRuntime.addShutdownHook(new Thread(() =>
    LOG.info("Server shutdowns")
  ))
