package org.tbasket.api

import org.apache.logging.log4j.LogManager
import org.tbasket.api.Endpoint.LOG
import org.tbasket.api.compute.APIRequestHandler
import zio.*
import zio.http.ServerConfig.LeakDetectionLevel
import zio.http.*
import zio.http.model.Method.{GET, POST}
import zio.http.model.Status

import scala.collection.mutable

class Endpoint(port: Int):
    
    private val handlers = mutable.HashMap.empty[String, APIRequestHandler]
    
    def bind(path: String)(handler: APIRequestHandler): Unit =
        handlers.put(path, handler)
    
    // set generic required headers
    private def transform(response: Response): Response =
        response.withAccessControlAllowOrigin("*")
    
    private val app = Http.collectZIO[Request] {
        case GET -> _ / path if handlers.contains(path)  =>
            handlers(path).get.map(transform)
            
        case POST -> _ / path if handlers.contains(path) =>
            handlers(path).post.map(transform)
            
        case r@method -> path                            =>
            val ipInsights = r.remoteAddress
                .map(ip => s": request received from $ip.")
                .getOrElse("")
            LOG.error(
                s"Was unable to find a handler for request '$path' with method $method ${ipInsights}"
            )
            ZIO.succeed(Response(Status.NotFound))
    }
    
    val run = 
        val config = ServerConfig.default
            .port(port)
            .leakDetection(LeakDetectionLevel.PARANOID)
        
        val configLayer = ServerConfig.live(config)
        Server.install(app).flatMap { port =>
            LOG.info(s"Listening API entries on $port")
            ZIO.never
        }.provideSome(configLayer, Server.live)

object Endpoint:
    final val LOG = LogManager.getLogger("API")
